/* Sponsored by EMBED-IT OG - {m.presulli@embed-it.com,t.znidar@embed-it.com}
 * 
 *
 */

#define DPU_BSP
#include <dpu.h>

#include <stdlib.h>
#include <stdio.h>

#include <mcf5225x/fec.h>

#include <netif/etharp.h>
#include <lwip/tcpip.h>
#include <lwip/stats.h>
#include <lwip/snmp_msg.h>

/* Internal used types */

struct snmp_trap_dst
{
  /* destination IP address in network order */
  struct ip_addr dip;
  /* set to 0 when disabled, >0 when enabled */
  u8_t enable;
};

#if LWIP_SNMP
struct snmp_config {
u8_t contact[20],contact_len;
u8_t location[20],location_len;
u8_t descr[30],descr_len;
u8_t name[20],name_len;
u8_t authentraps;
struct snmp_obj_id objid;
};
#endif

/**
 * Initialize platform specific external PHY
 */
static void
phy_init(void)
{
	MCF_GPIO_PORTNQ |= MCF_GPIO_PORTNQ_PORTNQ7; /* get PHY out of reset, platform specific */
	
	MCF_FEC_EIR |= MCF_FEC_EIR_MII;
	
	#if LWIP_NETIF_LINK_CALLBACK
	usleep(2500000); /* ~ 2 seconds for autoneg of phy in case of using status an link callback functionality */
	#endif
	
#if 0 
  printf("received data (phy_id1): 0x%x\n",smi_read(PHY_ADDR,PHY_REG_ID1)); /* read phy-id as an example */
  printf("received data (phy_id2): 0x%x\n",smi_read(PHY_ADDR,PHY_REG_ID2));
#endif

}

/*
 * Wake up the program mainline when the lwIP TCP task is properly initialized.  Called by tcpip_thread when it completes its initialization, but in the current version of lwIP, not necessary.
*/

static void
tcpip_init_done(void *post_config)
{
#if LWIP_SNMP
struct snmp_config * snmp_config=post_config;

snmp_set_syscontact(snmp_config->contact,&snmp_config->contact_len);
snmp_set_syslocation(snmp_config->location,&snmp_config->location_len);
snmp_set_sysdesr(snmp_config->descr,&snmp_config->descr_len);
snmp_set_sysname(snmp_config->name,&snmp_config->name_len);
snmp_set_snmpenableauthentraps(&snmp_config->authentraps);
snmp_set_sysobjid(&snmp_config->objid);
snmp_init();
#endif
}

/* 
 * Program entry.  Set up the lwIP system and create the interfaces. 
 */

rtems_task 
dpu_init_communicate(rtems_task_argument ignored)
{
	extern void init_vector_table(void);
	init_vector_table();
	
	struct if_config if_config={ 	.flags=NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP | NETIF_FLAG_LINK_UP,
													.hwaddr_len=ETHARP_HWADDR_LEN,
													.mtu=MTU_SIZE,
													.hwaddr= {0,0x12,0x34,0x56,0x78,0x9a},
													.net_task=rtems_task_self(),
													.phy_init=phy_init,
													.name="e1"
												};
	
	struct netif if_netif;
	struct ip_addr if_addr, if_mask, if_gate;
	
	#if LWIP_SNMP
												
	struct snmp_config snmp_config={"admin",sizeof("admin")-1,
															"Vienna",sizeof("Vienna")-1,
															"platform",sizeof("platform")-1,
															"DPU",sizeof("DPU")-1,
															2, /*disabled*/
															{7, {1, 3, 6, 1, 4, 1, 35493}}, /*PEN OID*/
															};
	
	struct ip_addr trap_addr;
	IP4_ADDR(&trap_addr, 192,168,1,100); //prepare a trap destination
	extern struct snmp_trap_dst trap_dst[SNMP_TRAP_DESTINATIONS];
	memcpy(&trap_dst[0].dip,&trap_addr,sizeof(struct ip_addr));
	trap_dst[0].enable=1;
	#endif
												
	// Set up an IF interface.
	IP4_ADDR(&if_addr, 192,168,1,10);	// IP address
	IP4_ADDR(&if_mask, 255,255,255,0);	// netmask
	IP4_ADDR(&if_gate, 192,168,1,1);	// gateway
	
	printf("WORKSPACE SIZE: %lu\n",Configuration.work_space_size);
	
	tcpip_init(tcpip_init_done,
	#if LWIP_SNMP
	&snmp_config
	#else
	NULL
	#endif
	);	// callback not currently needed

	netif_add(&if_netif, &if_addr, &if_mask, &if_gate,&if_config,mcf5225xif_init, tcpip_input);	// add the netif, initialize the driver via the if_config instance and link a CB function for the regular TCP/IP traffic
	netif_set_default(&if_netif);	// Set the default route.  This is the interface used for packets destined to networks we're not directly connected to.
	netif_set_up(&if_netif); 			// Power up the device now
	printf("interface set up!\n");
   
   //~ stats_display();
   
	while(1) { // waiting for the event from RXF interrupt
		rtems_event_set rcv_event;
		rtems_event_receive(RTEMS_EVENT_0,RTEMS_DEFAULT_OPTIONS,0,&rcv_event);
		handle_rx_frame(&if_netif); /* handle frame! */
		rcv_event&=~RTEMS_EVENT_0;
		 //~ stats_display();
	}

}
